import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import '../dio_extend.dart';
import 'http_config.dart';

/// dio扩展
class EDio {
  String TAG = 'EDio ======> ';
  Dio _dio;
  static var config = new Config();
  static EDio _instance = EDio._internal();

  factory EDio() => _instance;
  SignStrategy _signStrategy;
  Dio dio() => _dio;

  ///通用全局单例，第一次使用时初始化
  EDio._internal() {
    if (null == _dio) {
      _dio = new Dio(new BaseOptions(
          baseUrl: config.baseUrl, connectTimeout: config.connectTimeout, receiveTimeout: config.receiveTimeout));
    }
  }

  /// GET请求方法
  /// @param {string} uri 请求地址
  /// @param {map} params 请求入参
  /// @param {Options} options 请求配置
  Future<Response> get<T>(String uri, {Map<String, dynamic> params, Options options}) =>
      _request(uri, queryParameters: params, options: options, method: 'GET');

  /// POST请求方法
  /// @param {string} uri  请求地址
  /// @param {map} params 请求入参
  /// @param {Options} options 请求配置
  Future<Response> post<T>(String uri, {Map<String, dynamic> params, Options options}) =>
      _request(uri, queryParameters: params, options: options, method: 'POST');

  Future<Response> request<T>(String uri, {Map<String, dynamic> params, Options options, String method}) =>
      _request(uri, queryParameters: params, options: options, method: method);

  /// 通用请求方法
  /// @param {string} uri  请求地址
  /// @param {map} queryParameters 请求入参
  /// @param {Options} options 请求配置
  /// @param {string} method  请求类型
  Future<Response> _request(String uri, {Map<String, dynamic> queryParameters, Options options, String method}) async {
    Options op = Options();
    if (options != null) {
      op = options;
    } else {
      op = Options();
      if (method != null) {
        op.method = method;
        if (method.toLowerCase() == 'post') {
          op.contentType = Headers.formUrlEncodedContentType;
        }
      } else {
        op.method = 'get';
      }
    }

    Map<String, dynamic> parameters = new Map();
    if (_signStrategy != null) {
      parameters.addAll(_signStrategy.sign(queryParameters));
    } else {
      parameters.addAll(queryParameters);
    }

    debugPrint('$TAG _request ==== parameters = $parameters headers=${op.headers.toString()}');

    return await _dio.request(uri, queryParameters: parameters, options: op);
  }

  /// 设置接口签名策略
  void setStrategy(SignStrategy signStrategy) {
    _signStrategy = signStrategy;
  }

}
